unit HighPrecisionTimer;
(** /////////////////////////////////////////////////////////////////////////**)
(**                                                                          **)
(** For detailed information see the included html file                      **)
(**                                                                          **)
(** HighPrecision Timer was created by                                       **)
(**                                                                          **)
(** (c) Tom Peiffer, 1999 - peiffer@rhrk.uni-kl.de                           **)
(**                                                                          **)
(** and is based on the idea of using the processor speed for time           **)
(** measuring, copyright by                                                  **)
(**                                                                          **)
(** (c) NPS, 1997 	- Idea, realisation, adoptation IsHPTimer_Available) **)
(**                       kvk@estpak.ee                                      **)
(** (c) Alex Abreu, 1997 - IsHPTimer_Available source and idea               **)
(**                        simonet@bhnet.com.br                              **)
(** Gordon Bamber,June1998 Converted into a class + VCL                      **)
(**                        gbamber@mistral.co.uk                             **)
(**                                                                          **)
(** This code is presented 'as-is' and is not a commercial product.          **)
(** It compiles under Delphi 4.0 and is lightly, though adequately commented **)
(** Please do not distribute variants without including, and adding to this  **)
(** header information, and incrementing C_VERSIONSTRING		     **)
(** VERSION HISTORY							     **)
(** Version 1.0 Gordon Bamber						     **)
(**		VCL coded						     **)
(** Version 1.1 Gordon Bamber						     **)
(**		Public methods expanded to use TimeStamp more easily         **)
(** Version 1.2 Tom Peiffer                                                  **)
(**             a thread driven high precision timer, depending on processor **)
(**             speed                                                        **)
(** /////////////////////////////////////////////////////////////////////////**)

interface
uses Windows,SysUtils,Dialogs,Classes,DsgnIntf,Registry, thHPTimer ;

const
 C_DLLNAME='HPTimer';
 C_VERSIONSTRING='1.2';


 ID_BIT=$200000; (** EFLAGS ID bit **)
 C_SHOWWHOLESECONDSONLY=TRUE;
 C_DONTSHOWWHOLESECONDSONLY=FALSE;
 C_DISPLAYALL=TRUE;
 C_DONTDISPLAYALL=FALSE;
 type
 THPTimerARRAY=array[1..4] of Longint;
 TVendor=array [0..11] of char;
type
    TLargeInteger = record
    case integer of
      0 : (Lowpart : DWord ; HighPart : LongInt) ;
      1 : (QuadPart : LongLong) ;
    end ;

    TOnHPTimer = procedure (seconds : extended; Delta : extended;
                                  N : int64) of object ;

  THPTimer = class(TComponent)
  private
    { Private declarations }

   HPThread : THPTimerThread ;
   FAbout:string;
   fsVersion:String;
   fIDAvailable:Boolean;
   fId1:LongInt;
   fId2:LongInt;
   fId3:LongInt;
   fId4:LongInt;
   fProcessorType:LongInt;
   fFamily:LongInt;
   fModel:LongInt;
   fStepping:LongInt;
   fVendor:TVendor;
   fsVendor:String;
   fCPUSpeed:Comp;
   fCPUSpeedMhz:Double;
   fExactCPUSpeed:Double;
   fTicksPerSecond : double  ;

   fsIntelModel:String;
   fTimeStamp1:Word;
   fTimeStamp2:Word;
   fTimeStamp3:Word;
   fTimeStamp4:Word;

   fHasFPU:Boolean;
   fHasTimeStamp:Boolean;
   fHasAPIC:Boolean;


   (** Persistent stuff **)
   fStartTime,fEndTime,fElapsed:TLargeInteger;

   (** Dummies created for Property.Write values **)
   fDummyLongInt:LongInt;
   fDummyString:String;
   fDummyWord:Word;
   fDummyBool:Boolean;
   fDummyDouble:Double;
    FFrequency: extended;
    FTimePeriod: extended;
    FThreadPriority: TTHreadPriority;
    FEnabled: boolean;
    FOnHighPrecisionTimer: TOnHPTimer;
    FVersion: string;

   Property TimeStampHiHi:Word
        read fTimeStamp1 write fDummyWord stored False;

   Property TimeStampHiLo:Word
    read fTimeStamp2 write fDummyWord stored False;

   Property TimeStampLoHi:Word
    read fTimeStamp3 write fDummyWord stored False;

   Property TimeStampLoLo:Word
    read fTimeStamp4 write fDummyWord stored False;



   Function RegGetCPUSpeed:Comp;
   Function GetIDInfo:Boolean;
    procedure SetFrequency(const Value: extended);
    procedure SetTimePeriod(const Value: extended);
    procedure SetThreadPriority(const Value: TTHreadPriority);
    procedure SetEnabled(const Value: boolean);
    procedure SetOnHighPrecisionTimer(const Value: TOnHPTimer);
    procedure SetVersion(const Value: string);

  protected
    { Protected declarations }
  public
    { Public declarations }
  Procedure ShowAbout;Virtual;
  Constructor Create(aOwner:TComponent);override;
  Destructor Destroy ; override ;
  Procedure RefreshTimeStamp;
  Function GetTimeStamp:TLargeInteger;
  Procedure Loaded;override;

  Function SupportsTimeStamp:Boolean;
  Procedure StartTimer;
  Procedure StopTimer;
  Function GetElapsedTicks:Comp;
  Function GetTicksPerSecond(ITERATIONS:Word):Comp;
  function GetExactCpuSpeed:Double;

  Function GetElapsedSeconds:Double;
  Function GetCPUSpeed:Comp;

  published
    { Published declarations }
   Property About: string read FAbout write FAbout stored False;

   Property ProcessorSpeed:Double
    read fCPUSpeedMHz write fDummyDouble stored False;

   Property HasTimeStamp:Boolean
    read fHasTimeStamp write fDummyBool stored False;

   property Frequency : extended  read FFrequency write SetFrequency;
   property TimePeriod : extended  read FTimePeriod write SetTimePeriod;
   property ThreadPriority : TTHreadPriority  read FThreadPriority write SetThreadPriority;
   property Enabled : boolean  read FEnabled write SetEnabled;
   property Version : string  read FVersion write SetVersion;
   property OnHighPrecisionTimer : TOnHPTimer  read FOnHighPrecisionTimer write SetOnHighPrecisionTimer;
  end;

procedure Register;
(** These functions cannot be members of the class **)
function IsHPTimer_Available : Boolean;register;
function GetHPTimer : THPTimerARRAY;register;
function OldGetCpuSpeed: Comp;
Function GetTimeStampHi:DWord;register;
Function GetTimeStampLo:DWord;register;
function check_8086:Word;register;
function check_80286:Word;register;
function check_80386:Word;register;
function GetHPTimer4 :DWORD;register;
implementation

(** Keep some internal variables to save re-calculating all the time **)

(******************************************************************************)

type
  TAboutProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue:string; override;
  end;

(******************************************************************************)

procedure Register;
begin
  RegisterComponents('System', [THPTimer]);
  RegisterPropertyEditor(TypeInfo(String), THPTimer, 'About',
	TAboutProperty);
end;

(******************************************************************************)

procedure TAboutProperty.Edit;
{call the 'About' dialog window when clicking on ... in the Object Inspector}
begin
  THPTimer(GetComponent(0)).ShowAbout;
end;

(******************************************************************************)

function TAboutProperty.GetAttributes: TPropertyAttributes;
{set up to display a string in the Object Inspector}
begin
  GetAttributes := [paDialog, paReadOnly];
end;

(******************************************************************************)

function TAboutProperty.GetValue: String;
{set string to appear in the Object Inspector}
begin
  GetValue := '(About)';
end;

(******************************************************************************)

procedure THPTimer.ShowAbout;
var
	msg: string;
const
  cr = chr(13);
begin
   Msg := 'THPTimer (c) 1999' + cr +cr +
          'Tom Peiffer <peiffer@rhrk.uni-kl.de' + cr +
          '  a threaded High precision timer, based on idea of' + cr +
          'Gordon Bamber <gbamber@mistral.co.uk>,' + cr +
          'Alex Abreu <simonet@bbnet.com.br> and' + cr +
          'NPS <kvk@estpak.ee>' + cr +
          'who used the processor speed to make time measures' ;

   ShowMessage (Msg) ;
end;

(******************************************************************************)


Function THPTimer.GetIDInfo:Boolean;
var
  I     : Integer;
  HPTimerARRAY:THPTimerARRAY;
begin
Result:=False;
for I := Low(HPTimerARRAY) to High(HPTimerARRAY)  do HPTimerARRAY[I] := -1;
if IsHPTimer_Available then
 begin
  HPTimerARRAY:=GetHPTimer;

  fID1:=HPTimerARRAY[1];
  fID2:=HPTimerARRAY[2];
  fID3:=HPTimerARRAY[3];
  fID4:=HPTimerARRAY[4];

  fProcessorType:=(HPTimerARRAY[1] shr 12 and 3);
  fFamily:=(HPTimerARRAY[1] shr 8 and $f);
  fModel:=(HPTimerARRAY[1] shr 4 and $f);
  fStepping:=(HPTimerARRAY[1] and $f);
  fsVendor:=String(fVendor);
  Result:=True;
 end;
end;


Constructor THPTimer.Create(aOwner:TComponent);
begin
Inherited Create(aOwner);

FFrequency := 1 ;
FTimePeriod := 1 ;

fThreadPriority := tpLowest ;
FEnabled := false ;
hpThread := THPTimerThread.create (Self,fThreadPriority,FTimePeriod) ;
hpThread.TimePeriod := FTimePeriod ;


(** Initialise private members **)
FVersion := C_VERSIONSTRING ;
fTimeStamp1:=0;
fTimeStamp2:=0;
fTimeStamp3:=0;
fTimeStamp4:=0;
fCPUSpeedMhz:=0;
fCPUSpeed:=0;
fExactCPUSpeed:=0;

fIDAvailable:=GetIDInfo;
if (fIDAvailable) then
 begin
  (** Test bit flags of ID4 **)
  fHasFPU:=(fID4 and 1)=1;
  fHasTimeStamp:=(fID4 and 16)=16;
  fHasAPIC:=(fID4 and 512)=512;
  if fHasTimeStamp then
   begin
    RefreshTimeStamp; (** Sets the TimeStamp members **)
    if fCPUSpeed=0 then GetCPUSpeed;
   end;
 end; (** Of if fIDAvailable.. **)

(** Alternative methods for CPU Speed **)
if fCPUSpeed=0 then fCPUSpeed:=RegGetCPUSpeed; (** Works with NT **)
if fCPUSpeed=0 then OldGetCPUSpeed; (** Last ditch try! **)

(** If GetTicksPerSecond was called then fExactCPUSpeed
    was set, and this is the best available value **)
if fExactCPUSpeed > 0 then fCPUSpeedMhz:=fExactCPUSpeed
else(** Use fCPUSpeed, but convert cycles to MegaCycles **)
 if fCPUSpeed > 0 then fCPUSpeedMhz:=fCPUSpeed/1E6;
end;

(******************************************************************************)

Procedure THPTimer.Loaded;
begin
Inherited Loaded;
end;

(******************************************************************************)


Function GetTimeStampHi:DWord;assembler;register;
asm
 DW      $310F       {RDTSC Command}
 MOV @Result, EDX;
end;

(******************************************************************************)

Function GetTimeStampLo:DWord;assembler;register;
asm
  DW      $310F       {RDTSC Command}
  MOV @Result, EAX;
end;

(******************************************************************************)

Procedure THPTimer.RefreshTimeStamp;
begin
if fHasTimeStamp then
 begin
  fTimeStamp1:=HiWord(GetTimeStampHi);
  fTimeStamp2:=LoWord(GetTimeStampHi);
  fTimeStamp3:=HiWord(GetTimeStampLo);
  fTimeStamp4:=LoWord(GetTimeStampLo);
 end;
end;

(******************************************************************************)



Function THPTimer.RegGetCPUSpeed:Comp;
Var
 MyReg:TRegistry;
 B:LongInt;
 Mhz:LongInt;
begin
Result:=0;
MHz:=0;
TRY
 MyReg:=TRegistry.Create;
 MyReg.RootKey:=HKEY_LOCAL_MACHINE;
 TRY
 if MyReg.OpenKey('\HARDWARE\DESCRIPTION\System\CentralProcessor\0',FALSE) then
  Begin
   B:=MyReg.ReadInteger('~MHz');
   if B > 0 Then Mhz:=B else
    B:=MyReg.ReadInteger('~Mhz');
    if B > 0 Then Mhz:=B else
    B:=MyReg.ReadInteger('~mhz');
    if B > 0 Then Mhz:=B;
  End;
 FINALLY
  MyReg.Free;
 END;
EXCEPT
 Result:=0;
END;
Result:=Mhz * 1E6;
fCPUSpeed:=Result;
end;

(******************************************************************************)

function GetHPTimer4 :DWORD; assembler;register;
(** This function retrieves a set of HPTimer flags,
    one of which, is whether the TimeStamp counter exists **)
asm
  PUSH    EBX		(** Save registers **)
  PUSH    EDI
  MOV     EAX,1 	(** Set up for HPTimer **)
  DW      $A20F 	(** HPTimer OpCode **)
  MOV @Result,EDX	(** Put the flag array into a DWord **)
  POP     EDI		(** Restore registers **)
  POP     EBX
end;

(******************************************************************************)

Function THPTimer.SupportsTimeStamp:Boolean;
begin
Result:=FALSE;
If (GetHPTimer4 and 16) = 16 then result:=TRUE;
end;

(******************************************************************************)

Function THPTimer.GetTimeStamp:TLargeInteger;
begin
 Result.QuadPart:=0;
 If (GetHPTimer4 and 16) <> 16 then exit; (** No Timestamp Function **)
 Result.HighPart:=DWORD(GetTimeStampHi);
 Result.LowPart:=GetTimeStampLo;
End;

(******************************************************************************)

Procedure THPTimer.StartTimer;
Begin
 (** Get raw processor ticks since power-on **)
 fStartTime:=GetTimeStamp;
 fEndTime.QuadPart:=0;
 fElapsed.QuadPart:=0;
End;

(******************************************************************************)

Procedure THPTimer.StopTimer;
Begin
 (** Get raw processor ticks since power-on **)
 fEndTime:=GetTimeStamp;
 fElapsed.QuadPart:=(fEndTime.QuadPart-fStartTime.QuadPart);
End;

(******************************************************************************)

Function THPTimer.GetElapsedTicks:Comp;
Begin
 (** Implicit 64-bit TypeCast **)
 //Result:=fElapsed.QuadPart;
 fElapsed.QuadPart := GetTimeStamp.QuadPart - fStartTime.QuadPart ;
 Result := fElapsed.QuadPart ;
End;

(******************************************************************************)

Function THPTimer.GetTicksPerSecond(ITERATIONS:Word):Comp;
Var
 TARGET,PERFCOUNT,Freq:TLARGEINTEGER;
begin
  Result:=0; (** Initialise in case of failure **)

  if NOT QueryPerformanceFrequency(int64(FREQ)) then exit;
  (** The OS or hardware may not support this API call.
      Sometimes, the FREQ is equal to Processor Ticks/Second,
      but it might not be on every system **)

  QueryPerformanceCounter(int64(PERFCOUNT));
  (** Get a start time via the OS **)

  TARGET.QuadPart:=PERFCOUNT.QuadPart + (FREQ.QuadPart * ITERATIONS);
  (** Make an end time **)

  StartTimer;
  (** Fetch the processor tickcount **)


  REPEAT (** Loop for exactly one second (according to the OS) **)
   QueryPerformanceCounter(int64(PERFCOUNT));
   UNTIL (PERFCOUNT.QuadPart >= TARGET.QuadPart);

  (** Fetch the number of processor ticks since the loop started **)
  StopTimer;
  Result:=(fElapsed.QuadPart/ITERATIONS);
  fTicksPerSecond := Result ;

 (** In MegaHertz **)
 fExactCPUSpeed:=Result/1E6;
end;

(******************************************************************************)

function THPTimer.GetExactCpuSpeed:Double;
Var
 MYCOMP:Comp;
begin
 (** Do a 10-second test **)
 MYCOMP:=GetTicksPerSecond(10);
 Result:=MYCOMP/1E6;

 (** In MegaHertz **)
 fExactCPUSpeed:=Result;
end;

(******************************************************************************)


Function THPTimer.GetElapsedSeconds:Double;
Begin
 //Result:=GetElapsedTicks/(fCPUSpeed);
 Result:=GetElapsedTicks/(fTicksPerSecond);
End;

(******************************************************************************)

Function THPTimer.GetCPUSpeed:Comp;
Begin
 if fCPUSpeed=0 then fCPUSpeed:=GetTicksPerSecond(1);
 Result:=fCPUSpeed;
End;

(******************************************************************************)

(** This function is not as accurate **)
function OldGetCpuSpeed: Comp;
(** Uses RDTSC Command to get the Processor Timer count **)
var
  t: DWORD;
  mhi, mlo, nhi, nlo: DWORD;
  t0, t1, chi, clo, shr32: Comp;
begin
Result:=0;
  shr32 := 65536;
  shr32 := shr32 * 65536;

  (** Get a start time **)
  t := GetTickCount;
  while t = GetTickCount do begin end;
  asm
    DB 0FH
    DB 031H  	(** RDTSC Opcode retrieves 64-bit tick count since boot **)
    mov mhi,edx (** 32-bit Ticks Hi **)
    mov mlo,eax (** 32-bit Ticks Lo **)
  end;

 (** Look 1 second later **)
  while GetTickCount < (t + 1000) do begin end;
  asm
    DB 0FH
    DB 031H  (** RDTSC **)
    mov nhi,edx
    mov nlo,eax
  end;

  (** See how many Processor Timer Ticks have elapsed **)
  chi := mhi; if mhi < 0 then chi := chi + shr32;
  clo := mlo; if mlo < 0 then clo := clo + shr32;

  t0 := chi * shr32 + clo;

  chi := nhi; if nhi < 0 then chi := chi + shr32;
  clo := nlo; if nlo < 0 then clo := clo + shr32;

  t1 := chi * shr32 + clo;

  Result := Round(t1 - t0);
end;

(******************************************************************************)

{Function THPTimer.GetIDInfo:Boolean;
var
  I     : Integer;
  HPTimerARRAY:THPTimerARRAY;
begin
Result:=False;
for I := Low(HPTimerARRAY) to High(HPTimerARRAY)  do HPTimerARRAY[I] := -1;
if IsHPTimer_Available then
 begin
  HPTimerARRAY:=GetHPTimer;

  fID1:=HPTimerARRAY[1];
  fID2:=HPTimerARRAY[2];
  fID3:=HPTimerARRAY[3];
  fID4:=HPTimerARRAY[4];

  fProcessorType:=(HPTimerARRAY[1] shr 12 and 3);
  fFamily:=(HPTimerARRAY[1] shr 8 and $f);
  fModel:=(HPTimerARRAY[1] shr 4 and $f);
  fStepping:=(HPTimerARRAY[1] and $f);
  fVendor := GetCPUVendor;
  fsVendor:=String(fVendor);
  Result:=True;
 end;
end;}

(******************************************************************************)

function IsHPTimer_Available : Boolean;assembler;register;
asm
  PUSHFD							{direct access to flags no possible, only via stack}
  POP     EAX					{flags to EAX}
  MOV     EDX,EAX			{save current flags}
  XOR     EAX,ID_BIT	{not ID bit}
  PUSH    EAX					{onto stack}
  POPFD								{from stack to flags, with not ID bit}
  PUSHFD							{back to stack}
  POP     EAX					{get back to EAX}
  XOR     EAX,EDX			{check if ID bit affected}
  JZ      @exit				{no, HPTimer not availavle}
  MOV     AL,True			{Result=True}
@exit:
end;

(******************************************************************************)

function GetHPTimer : THPTimerARRAY; assembler;register;
asm
  PUSH    EBX         	{Save affected register}
  PUSH    EDI
  MOV     EDI,EAX     	{@Resukt}
  MOV     EAX,1
  DW      $A20F       	{HPTimer Command}
  STOSD			{HPTimer[1]}
  MOV     EAX,EBX
  STOSD               	{HPTimer[2]}
  MOV     EAX,ECX
  STOSD               	{HPTimer[3]}
  MOV     EAX,EDX
  STOSD               	{HPTimer[4]}
  POP     EDI					{Restore registers}
  POP     EBX
end;

(******************************************************************************)


(******************************************************************************)

function check_8086 : Word; assembler;register;
(** Returns:
   0      if processor 8086
   0xffff otherwise
**)
asm
  pushf;                   (** Push original FLAGS **)
  pop     ax;              (** Get original FLAGS **)
  mov     cx, ax;          (** Save original FLAGS **)
  and     ax, $0fff;       (** Clear bits 12-15 in FLAGS **)
  push    ax;              (** Save new FLAGS value on stack **)
  popf;                    (** Replace current FLAGS value **)
  pushf;                   (** Get new FLAGS **)
  pop     ax;              (** Store new FLAGS in AX **)
  and     ax, $0f000;      (** If bits 12-15 are set, then **)
  cmp     ax, $0f000;      (**   processor is an 8086/8088 **)
  mov     @Result, 0;      (** Turn on 8086/8088 flag **)
  je	  @is8086;  	   (** Jump if processor is 8086 or 8088 **)
  mov @Result, $ffff;
@is8086:
  push cx;
  popf;
end;

(******************************************************************************)

function check_80286 : Word; assembler;register;
(** Returns:
   2      if processor 80286
   0xffff otherwise
**)
asm
  pushf;
  pop cx;
  mov bx, cx;
  or cx, $0f000;	(** Try to set bits 12-15 **)
  push cx;		(** Save new FLAGS value on stack **)
  popf;			(** Replace current FLAGS value **)
  pushf;			(** Get new FLAGS **)
  pop ax;		(** Store new FLAGS in AX **)
  and ax, $0f000;	(** If bits 12-15 are clear **)
  mov @Result, 2;	(** Processor=80286, turn on 80286 flag **)
  jz @is80286;		(** If no bits set, processor is 80286 **)
  mov @Result, $0ffff;
@is80286:
  push	bx;
  popf;
end;

(******************************************************************************)

function check_80386 : Word; assembler;register;
(** Returns:
   3      if processor 80386
   0xffff otherwise
**)
asm
 mov bx, sp;
 and sp, not 3;
 pushfd;		(** Push original EFLAGS **)
 pop eax;		(** Get original EFLAGS **)
 mov ecx, eax;		(** Save original EFLAGS **)
 xor eax, $40000;	(**  Flip AC bit in EFLAGS **)
 push eax;		(** Save new EFLAGS value on stack **)
 popfd;			(** Replace current EFLAGS value **)
 pushfd;		(** Get new EFLAGS **)
 pop eax;		(** Store new EFLAGS in EAX **)
 xor eax, ecx;		(** Can't toggle AC bit, so processor=80386 **)
 mov @Result, 3;	(** Turn on 80386 processor flag **)
 jz  @is80386;		(** Jump if 80386 processor **)
 mov @Result, $0ffff;
@is80386:
 push	ecx;
 popfd;
 mov sp, bx;		(** Restore SP **)
 and eax, $0000ffff;
end;
(******************************************************************************)



procedure THPTimer.SetFrequency(const Value: extended);
begin
  FFrequency := Value;
  FTimePeriod := 1/Value ;
  hpThread.TimePeriod := FTimePeriod ;
end;

procedure THPTimer.SetTimePeriod(const Value: extended);
begin
  FTimePeriod := Value;
  FFrequency := 1/Value ;
  hpThread.TimePeriod := FTimePeriod ;
end;

procedure THPTimer.SetThreadPriority(const Value: TTHreadPriority);
begin
  FThreadPriority := Value;
  hpThread.Priority := Value ;
end;

procedure THPTimer.SetEnabled(const Value: boolean);
begin
  FEnabled := Value;
  hpThread.enabled := value ;
  if Value
  then hpThread.resume
  else hpThread.suspend ;
end;

procedure THPTimer.SetOnHighPrecisionTimer(const Value: TOnHPTimer);
begin
  FOnHighPrecisionTimer := Value;
end;

destructor THPTimer.Destroy;
begin
     enabled := false ;
     hpThread.free ;
     inherited Destroy ;
end;

procedure THPTimer.SetVersion(const Value: string);
begin
  FVersion := C_VERSIONSTRING ;
end;

end.

