unit thHPTimer;

{ (c) Tom Peiffer }
{ This is the threaded part which triggers the OnHighPrecisionTimer event }
{ definde in HighPrecisionTimer.pas }

interface
uses classes, extctrls, windows ;

type
 THPTimerThread = class (TThread)
 private
   i : integer ;
   s, olds : extended ;
   FOwner : TComponent ;
    NTimes : int64 ;
    FTimePeriod: extended;
    FEnabled: boolean;
    procedure SetTimePeriod(const Value: extended);
    procedure SetEnabled(const Value: boolean);
 protected
   procedure Execute ; override ;
   procedure DoTimerWatch ;
 public
   property TimePeriod : extended  read FTimePeriod write SetTimePeriod;
   property Enabled : boolean  read FEnabled write SetEnabled;
   constructor Create (AOwner : TComponent ;ThreadPriority : TThreadPriority;
                      TP : extended) ;
 end ;

implementation
uses HighPrecisionTimer, dialogs, sysutils ;

{ THPTimerThread }

constructor THPTimerThread.Create(AOwner : TComponent ; ThreadPriority: TThreadPriority ;
                                           TP : extended);
begin
 inherited Create (false) ;
 FOwner := AOwner ;
 Enabled := (FOwner as THPTimer).Enabled ;
 FTimePeriod := TP ;
 Priority := ThreadPriority ;
end;

procedure THPTimerThread.DoTimerWatch;
begin
     if Assigned((FOwner as THPTimer).OnHighPrecisionTimer)
     then (FOwner as THPTimer).OnHighPrecisionTimer (s,s-olds,NTimes) ;
     olds := s ;
     NTimes := NTimes + 1 ;
end;

procedure THPTimerThread.Execute;
begin
   while true do begin
       repeat
         s := (FOwner as THPTimer).GetElapsedSeconds ;
       until s >= (NTimes+1) * FTimePeriod ;
       if FEnabled then Synchronize (DoTimerWatch) ;
   end ;
end;


procedure THPTimerThread.SetEnabled(const Value: boolean);
begin
     if Value
     then begin
             NTimes := 1 ;
             (FOwner as THPTimer).StartTimer ;
             repeat
             until (FOwner as THPTimer).GetElapsedSeconds >= FTimePeriod ;
             Olds := (FOwner as THPTimer).GetElapsedSeconds ;
             FEnabled := Value ;
          end
     else FEnabled := Value ;
end;

procedure THPTimerThread.SetTimePeriod(const Value: extended);
begin
  FTimePeriod := Value;
end;

end.
